@@ -1,3 +1,5 @@
+import os
+
 from .setting import Setting
 from ..application.futures import HttpRequest, HttpResponse
 from ..application.tools import pprint
@@ -31,17 +33,29 @@ class Container(Base):
     mapping: 保存所有controller
     filter: 保存所有filter
     reject: 是一个列表，ip地址拦截队列
+    -----------------------------------------------------------------
+    0.3.0 修改
+    添加 resource 属性
+    修改了init_mapping方法
+    -----------------------------------------------------------------
     """
     def __init__(self):
         self.mapping = ControllerMapping()
         self.filter = FilterChain()
         self.reject = None
+        self.resource = Resource()
 
     def init_mapping(self, setting: Setting):
         """
-        0.2.3新增
+        -----------------------------------------------------------------
+        0.2.3 新增
         将原来的init()函数拆分，只保留了mapping和reject的初始化
         拆分是为了能够确保add_router添加的mapping和filter在对应对象初始化后分别正确的添加
+        -----------------------------------------------------------------
+        0.3.0 修改
+        添加对resource对象的初始化
+        以及将资源文件添加进mapping中
+        -----------------------------------------------------------------
         :param setting:
         :return:
         """
@@ -52,6 +66,11 @@ class Container(Base):
         # 初始化拦截队列
         self.reject = setting.container_reject
 
+        # 初始化资源文件
+        self.resource.init(setting)
+        for path in self.resource.res_path:
+            self.mapping.add(path, self.resource.get_resource, 'all')
+
     def init_filter(self, setting: Setting):
         """
         0.2.3新增
@@ -739,3 +758,98 @@ class OrderDic:
             st += ("'"+i+"'" if isinstance(i, str) else str(i)) + ": " + \
                   ("'"+value+"'" if isinstance(value, str) else str(value))+","
         return st[:-1]+"} "
+
+
+class Resource(Base):
+    """
+    0.3.0 添加
+    实现对资源文件的扫描
+    res_path 资源文件
+        数据保存方式
+        {path1: name1, path2: name2}
+        说明：
+            path: 加载到mapping后的名称或者叫path，例如：/index.html
+            name：资源文件的名称，在计算机中的相对路径，例如：resource/index.html
+    """
+
+    def __init__(self):
+        self.res_path = {}
+
+    def init(self, setting: Setting):
+        """
+        扫描所有资源文件，添加进res_path
+        :param setting:
+        :return:
+        """
+        for file in Setting.container_resources:
+            if os.path.isdir(file):
+                for name in Resource.get_all_files(file):
+                    path = name[name.find('/'):]
+                    if path.find("/index.") >= 0:
+                        self.res_path.update({'/': name})
+                    self.res_path.update({path: name})
+
+    def get_resource(self, request: HttpRequest) -> HttpResponse:
+        """
+        这个函数将被作为值添加到mapping中
+        资源文件被引用时，run_function函数将调用它
+        获取文件访问路径并打包资源文件
+        :param request:
+        :return:
+        """
+        path = request.path
+        if path in self.res_path:
+            return Resource.get_resource_response(self.res_path[path])
+        return HttpResponse(data="this resource is not Found!", status=404)
+
+    @staticmethod
+    def get_all_files(dir_: str) -> list:
+        """
+        获取文件夹下所有文件的名字
+        :param dir_: 文件夹名
+        :return: list
+        """
+        file_list = []
+        files = os.listdir(dir_)
+        for file in files:
+            file = dir_ + "/" + file
+            if os.path.isdir(file):
+                file_list += Resource.get_all_files(file)
+            else:
+                file_list.append(file)
+        return file_list
+
+    @staticmethod
+    def get_resource_response(file_path: str) -> HttpResponse:
+        """
+        将资源文件打包为Response对象
+        :param file_path:
+        :return:
+        """
+        file_data = b""
+        for data in Resource.file_iterator(file_path):
+            file_data += data
+
+        response = HttpResponse(file_data)
+
+        file_paths = file_path.split('/')
+        index = file_paths[-1].rfind('.')
+        if index > 0:
+            response.set_content_type(file_paths[-1][index+1:])
+        return response
+
+    @staticmethod
+    def file_iterator(file_path, chunk_size=512):
+        """
+        文件读取迭代器
+        :param file_path:文件路径
+        :param chunk_size: 每次读取流大小
+        :return: itr
+        """
+        with open(file_path, 'rb') as target_file:
+            while True:
+                chunk = target_file.read(chunk_size)
+                if chunk:
+                    yield chunk
+                else:
+                    break
